/* Miscellaneous support functions */


#include <malloc.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <misc.h>


void *safe_malloc(size_t size)
{
    static char msg[] = "FATAL: Out of memory!\n";
    void *p;

    if ((p = malloc(size))) return p;
    write(2, msg, sizeof(msg));
    _exit(255);
}


/* A read that never returns -1. EINTR is handled, the rest ends the program. 
   Not suitable for nonblocking I/O, unless you always select() first. */

int safe_read(int fd, void *p, size_t l)
{
    static char msg[] = "FATAL: I/O error: ";
    char *s;
    int ret;

    do {
	ret = read(fd, p, l);
	if (ret != -1) return ret;
    } while (errno == EINTR);
    s = strerror(errno); write(2, msg, sizeof(msg)); 
    write(2, s, strlen(s)); write(2, "\n", 1);
    _exit(1);
}


void hex(char *buf, const char *src, ssize_t len)
{
    static char *hextbl = "0123456789abcdef";

    for( ; len > 0; len--) {
	*buf++ = hextbl[(*src >> 4) & 0xf];
	*buf++ = hextbl[*src++ & 0xf];
    }
}


/* Get reasonably good random data */

void get_random_data(char *p, ssize_t len)
{
#ifdef RANDOM_DEV
    static char msg[] = "WARNING: No data from " RANDOM_DEV ", using rand()\n";
    int fd;
    ssize_t n;

    if ((fd = open(RANDOM_DEV, O_RDONLY)) != -1 &&
	(n = read(fd, p, len)) == len &&
	close(fd) != -1) return;

    write(2, msg, sizeof(msg));
#endif
    while(len-- > 0) *p++ = (char)(256.0 * rand() / (RAND_MAX + 1.0));
}


/*
 * vim:softtabstop=4:sw=4
 */
